{{ define "main" }}
<article class="markdown book-post">
  <h1>{{ .Title | title }}</h1>
  {{ $taxonomies := index .Site.Taxonomies .Page.Type }}
  {{ range $taxonomies }}
    <div><a href="{{ .Page.Permalink }}">{{ .Page.Title }} <sup>{{ .Count }}</sup></a></div>
  {{ end }}
</article>
{{ end }}

{{ define "toc" }}
  {{ partial "docs/taxonomy" . }}
{{ end }}
