{{ define "main" }}
<article class="markdown book-post">
  <h1>
    {{ partial "docs/title.html" . }}
  </h1>
  {{ partial "docs/post-meta" . }}
  <div class="book-post-content">
    {{- .Content -}}
  </div>
</article>
{{ end }}

{{ define "toc" }}
  {{ partial "docs/toc" . }}
{{ end }}
