<details {{ if or (.Get "open") (in .Params "open") }}open{{ end }}>
  {{- $summary := cond .IsNamedParams (.Get "title") (.Get 0) -}}
  <summary>{{ $summary }}</summary>
  <div class="markdown-inner">
    {{ .InnerDeindent | safeHTML }}
  </div>
</details>
