<h2 class="book-brand">
  <a class="flex align-center" href="{{ cond (not .Site.Home.File) .Sites.Default.Home.RelPermalink .Site.Home.RelPermalink }}">
    {{- with .Site.Params.BookLogo -}}
    <img src="{{ . | relURL }}" alt="Logo" class="book-icon" />
    {{- end -}}
    <span>{{ .Site.Title }}</span>
  </a>
</h2>
